namespace Tiles;

public class TileManager
{

    private Dictionary<string, int> _tileNameToId;
    private Dictionary<int, Tile> _tileIdToTile;
    
    public Dictionary<string, int> TileNameToIdList => _tileNameToId;
    
    public TileManager()
    {
        _tileNameToId = new Dictionary<string, int>();
        _tileIdToTile = new Dictionary<int, Tile>();
        
        SetupTiles();
    }
    
    public void SetupTiles()
    {
        _tileNameToId.Add("air", 0);

        int index = 1;
        
        foreach (var path in Directory.GetFiles("data/core/tiles/", "*.json"))
        {
            Tile tile;

            try
            {
                tile = Tile.GenerateTileFromJson(path);
        
                if (tile == null) throw new Exception($"Couldn't load tile {path}");
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
                throw;
            }
    
            Console.WriteLine($"Tile {tile.Name} loaded");
            _tileIdToTile.Add(index, tile);
            _tileNameToId.Add(tile.Name, index);
            index++;
        }
    }

    public Tile GetTileFromId(int id)
    {
        if (!_tileIdToTile.ContainsKey(id))
        {
            throw new Exception($"Tile {id} not found");
        }
        
        return _tileIdToTile[id];
    }

    public int GetTileIdFromName(string name)
    {
        if (!_tileNameToId.ContainsKey(name))
        {
            throw new Exception($"Tile {name} not found");
        }
    
        return _tileNameToId[name];
    }
}